@@ -10,19 +10,13 @@ from .fpdf import (
     FPDF_VERSION as _FPDF_VERSION,
 )
 from .html import HTMLMixin, HTML2FPDF
+from .prefs import ViewerPreferences
 from .template import Template, FlexTemplate
 from . import svg
 from .deprecation import WarnOnDeprecatedModuleAttributes
 
 FPDF_VERSION = _FPDF_VERSION
-"""Current FPDF Version, also available via `__version__` (which is read by `setup.py`):
-
-<pre>
->>> import fpdf
->>> fpdf.__version__
-'2.2.0'
-</pre>
-"""
+"Current FPDF Version, also available via `__version__`"
 
 FPDF_FONT_DIR = _FPDF_FONT_DIR
 """This is the location of where to look for fonts."""
@@ -45,6 +39,7 @@ __all__ = [
     "Template",
     "FlexTemplate",
     "TitleStyle",
+    "ViewerPreferences",
     "HTMLMixin",
     "HTML2FPDF",
     # FPDF Constants
