@@ -4,13 +4,6 @@ from sys import intern
 from .syntax import Name
 
 
-class DocumentState(IntEnum):
-    UNINITIALIZED = 0
-    READY = 1  # page not started yet
-    GENERATING_PAGE = 2
-    CLOSED = 3  # EOF printed
-
-
 class SignatureFlag(IntEnum):
     SIGNATURES_EXIST = 1
     "If set, the document contains at least one signature field."
