@@ -6,16 +6,19 @@ __license__ = "LGPL 3.0"
 
 # Inspired by tuto5.py and several examples from fpdf.org, html2fpdf, etc.
 
-import html
-import logging
+import logging, warnings
 from html.parser import HTMLParser
 
 from .enums import XPos, YPos
 
+import re
 
 LOGGER = logging.getLogger(__name__)
 BULLET_WIN1252 = "\x95"  # BULLET character in Windows-1252 encoding
 DEFAULT_HEADING_SIZES = dict(h1=24, h2=18, h3=14, h4=12, h5=10, h6=8)
+LEADING_SPACE = re.compile(r"^\s+")
+WHITESPACE = re.compile(r"(\s)(\s*)")
+TRAILING_SPACE = re.compile(r"\s$")
 
 COLOR_DICT = {
     "black": "#000000",
@@ -200,6 +203,7 @@ class HTML2FPDF(HTMLParser):
         pdf,
         image_map=None,
         li_tag_indent=5,
+        dd_tag_indent=10,
         table_line_separators=False,
         ul_bullet_char=BULLET_WIN1252,
         heading_sizes=None,
@@ -211,6 +215,7 @@ class HTML2FPDF(HTMLParser):
             image_map (function): an optional one-argument function that map <img> "src"
                 to new image URLs
             li_tag_indent (int): numeric indentation of <li> elements
+            dd_tag_indent (int): numeric indentation of <dd> elements
             table_line_separators (bool): enable horizontal line separators in <table>
             ul_bullet_char (str): bullet character for <ul> elements
         """
@@ -218,9 +223,13 @@ class HTML2FPDF(HTMLParser):
         self.pdf = pdf
         self.image_map = image_map or (lambda src: src)
         self.li_tag_indent = li_tag_indent
+        self.dd_tag_indent = dd_tag_indent
         self.table_line_separators = table_line_separators
         self.ul_bullet_char = ul_bullet_char
         self.style = dict(b=False, i=False, u=False)
+        self.pre_formatted = False
+        self.follows_fmt_tag = False
+        self.follows_trailing_space = False
         self.href = ""
         self.align = ""
         self.page_links = {}
@@ -261,6 +270,7 @@ class HTML2FPDF(HTMLParser):
         return int(length)
 
     def handle_data(self, data):
+        trailing_space_flag = TRAILING_SPACE.search(data)
         if self.td is not None:  # drawing a table?
             self._insert_td(data)
         elif self.table is not None:
@@ -278,15 +288,43 @@ class HTML2FPDF(HTMLParser):
                 align=self.align[0].upper(),
                 link=self.href,
             )
-        else:
-            data = data.replace("\n", " ")
+        elif self.pre_formatted:  # for pre blocks
+            self.pdf.write(self.h, data)
+
+        elif self.follows_fmt_tag and not self.follows_trailing_space:
+            # don't trim leading whitespace if following a format tag with no trailing whitespace
+            data = WHITESPACE.sub(whitespace_repl, data)
+            if trailing_space_flag:
+                self.follows_trailing_space = True
             if self.href:
                 self.put_link(data)
             else:
                 if self.heading_level:
                     self.pdf.start_section(data, self.heading_level - 1)
-                LOGGER.debug("write '%s' h=%d", data.replace("\n", "\\n"), self.h)
+                LOGGER.debug(
+                    "write '%s' h=%d",
+                    WHITESPACE.sub(whitespace_repl, data),
+                    self.h,
+                )
                 self.pdf.write(self.h, data)
+            self.follows_fmt_tag = False
+
+        else:
+            data = LEADING_SPACE.sub(leading_whitespace_repl, data)
+            data = WHITESPACE.sub(whitespace_repl, data)
+            self.follows_trailing_space = trailing_space_flag
+            if self.href:
+                self.put_link(data)
+            else:
+                if self.heading_level:
+                    self.pdf.start_section(data, self.heading_level - 1)
+                LOGGER.debug(
+                    "write '%s' h=%d",
+                    WHITESPACE.sub(whitespace_repl, data),
+                    self.h,
+                )
+                self.pdf.write(self.h, data)
+            self.follows_fmt_tag = False
 
     def _insert_td(self, data=""):
         self._only_imgs_in_td = False
@@ -439,6 +477,12 @@ class HTML2FPDF(HTMLParser):
     def handle_starttag(self, tag, attrs):
         attrs = dict(attrs)
         LOGGER.debug("STARTTAG %s %s", tag, attrs)
+        if tag == "dt":
+            self.pdf.ln(self.h)
+            tag = "b"
+        if tag == "dd":
+            self.pdf.ln(self.h)
+            self.pdf.write(self.h, " " * self.dd_tag_indent)
         if tag == "strong":
             tag = "b"
         if tag == "em":
@@ -464,9 +508,13 @@ class HTML2FPDF(HTMLParser):
                 self.align = attrs.get("align")
         if tag == "hr":
             self.pdf.add_page(same=True)
+        if tag == "code":
+            self.font_stack.append((self.font_face, self.font_size, self.font_color))
+            self.set_font("courier", 11)
         if tag == "pre":
             self.font_stack.append((self.font_face, self.font_size, self.font_color))
             self.set_font("courier", 11)
+            self.pre_formatted = True
         if tag == "blockquote":
             self.pdf.set_text_color(100, 0, 45)
             self.indent += 1
@@ -614,20 +662,26 @@ class HTML2FPDF(HTMLParser):
             self.set_font(face, size)
             self.set_text_color(*color)
             self.align = None
+        if tag == "code":
+            face, size, color = self.font_stack.pop()
+            self.set_font(face, size)
+            self.set_text_color(*color)
         if tag == "pre":
             face, size, color = self.font_stack.pop()
             self.set_font(face, size)
             self.set_text_color(*color)
+            self.pre_formatted = False
         if tag == "blockquote":
             self.set_text_color(*self.font_color)
             self.indent -= 1
             self.pdf.ln(3)
-        if tag == "strong":
+        if tag in ("strong", "dt"):
             tag = "b"
         if tag == "em":
             tag = "i"
         if tag in ("b", "i", "u"):
             self.set_style(tag, False)
+            self.follows_fmt_tag = True
         if tag == "a":
             self.href = ""
         if tag == "p":
@@ -678,8 +732,10 @@ class HTML2FPDF(HTMLParser):
             self.align = None
         if tag == "sup":
             self.pdf.char_vpos = "LINE"
+            self.follows_fmt_tag = True
         if tag == "sub":
             self.pdf.char_vpos = "LINE"
+            self.follows_fmt_tag = True
 
     def set_font(self, face=None, size=None):
         if face:
@@ -735,14 +791,39 @@ class HTML2FPDF(HTMLParser):
         raise RuntimeError(message)
 
 
-class HTMLMixin:
-    HTML2FPDF_CLASS = HTML2FPDF
+def leading_whitespace_repl(matchobj):
+    trimmed_str = ""
+    for char in matchobj.group(0):  # check if leading whitespace contains nbsp
+        if char == "\u00a0":
+            trimmed_str += "\u00a0"
+        elif char == "\u202f":
+            trimmed_str += "\u202f"
+    return trimmed_str
 
-    def write_html(self, text, *args, **kwargs):
-        """Parse HTML and convert it to PDF"""
-        kwargs2 = vars(self)
-        # Method arguments must override class & instance attributes:
-        kwargs2.update(kwargs)
-        h2p = self.HTML2FPDF_CLASS(self, *args, **kwargs2)
-        text = html.unescape(text)  # To deal with HTML entities
-        h2p.feed(text)
+
+def whitespace_repl(matchobj):
+    trimmed_str = ""
+    for char in matchobj.group(
+        1
+    ):  # allow 1 whitespace char, check for narrow no-break space
+        if char == "\u202f":
+            trimmed_str += "\u202f"
+        else:
+            trimmed_str += " "
+    for char in matchobj.group(2):  # remove following whitespace char unless nbsp
+        if char == "\u00a0":
+            trimmed_str += "\u00a0"
+        elif char == "\u202f":
+            trimmed_str += "\u202f"
+    return trimmed_str
+
+
+class HTMLMixin:
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        warnings.warn(
+            "The HTMLMixin class is deprecated. "
+            "Simply use the FPDF class as a replacement.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
