@@ -1,6 +1,6 @@
-import zlib
+import base64, zlib
 from io import BytesIO
-import base64
+from math import ceil
 from urllib.request import urlopen
 
 try:
@@ -70,12 +70,15 @@ def get_img_info(img, image_filter="AUTO", dims=None):
     if img.mode in ("P", "PA") and image_filter != "FlateDecode":
         img = img.convert("RGBA")
 
-    if img.mode not in ("L", "LA", "RGB", "RGBA", "P", "PA"):
+    if img.mode not in ("1", "L", "LA", "RGB", "RGBA", "P", "PA"):
         img = img.convert("RGBA")
 
     w, h = img.size
     info = {}
-    if img.mode == "L":
+    if img.mode == "1":
+        dpn, bpc, colspace = 1, 1, "DeviceGray"
+        info["data"] = _to_data(img, image_filter)
+    elif img.mode == "L":
         dpn, bpc, colspace = 1, 8, "DeviceGray"
         info["data"] = _to_data(img, image_filter)
     elif img.mode == "LA":
@@ -124,8 +127,6 @@ def get_img_info(img, image_filter="AUTO", dims=None):
         ):
             info["smask"] = _to_data(img, image_filter, select_slice=alpha_channel)
 
-    dp = f"/Predictor 15 /Colors {dpn} /BitsPerComponent {bpc} /Columns {w}"
-
     info.update(
         {
             "w": w,
@@ -133,8 +134,7 @@ def get_img_info(img, image_filter="AUTO", dims=None):
             "cs": colspace,
             "bpc": bpc,
             "f": image_filter,
-            "dp": dp,
-            "trns": "",
+            "dp": f"/Predictor 15 /Colors {dpn} /Columns {w}",
         }
     )
 
@@ -171,8 +171,11 @@ def _to_zdata(img, remove_slice=None, select_slice=None):
     if select_slice:
         data = data[select_slice]
     # Left-padding every row with a single zero:
-    channels_count = len(data) // (img.size[0] * img.size[1])
-    loop_incr = img.size[0] * channels_count + 1
+    if img.mode == "1":
+        loop_incr = ceil(img.size[0] / 8) + 1
+    else:
+        channels_count = len(data) // (img.size[0] * img.size[1])
+        loop_incr = img.size[0] * channels_count + 1
     i = 0
     while i < len(data):
         data[i:i] = b"\0"
